@@ -27,7 +27,7 @@
  * list of people on the gedit Team.  
  * See the ChangeLog files for a list of changes. 
  *
- * $Id: gedit-document.h 6398 2008-08-13 16:12:21Z johan $
+ * $Id$
  */
  
 #ifndef __GEDIT_DOCUMENT_H__
@@ -51,6 +51,29 @@ G_BEGIN_DECLS
 #define GEDIT_IS_DOCUMENT_CLASS(klass)   (G_TYPE_CHECK_CLASS_TYPE ((klass), GEDIT_TYPE_DOCUMENT))
 #define GEDIT_DOCUMENT_GET_CLASS(obj)    (G_TYPE_INSTANCE_GET_CLASS((obj), GEDIT_TYPE_DOCUMENT, GeditDocumentClass))
 
+#ifdef G_OS_WIN32
+#define GEDIT_METADATA_ATTRIBUTE_POSITION "position"
+#define GEDIT_METADATA_ATTRIBUTE_ENCODING "encoding"
+#define GEDIT_METADATA_ATTRIBUTE_LANGUAGE "language"
+#else
+#define GEDIT_METADATA_ATTRIBUTE_POSITION "metadata::gedit-position"
+#define GEDIT_METADATA_ATTRIBUTE_ENCODING "metadata::gedit-encoding"
+#define GEDIT_METADATA_ATTRIBUTE_LANGUAGE "metadata::gedit-language"
+#endif
+
+typedef enum
+{
+	GEDIT_DOCUMENT_NEWLINE_TYPE_LF,
+	GEDIT_DOCUMENT_NEWLINE_TYPE_CR,
+	GEDIT_DOCUMENT_NEWLINE_TYPE_CR_LF
+} GeditDocumentNewlineType;
+
+#ifdef G_OS_WIN32
+#define GEDIT_DOCUMENT_NEWLINE_TYPE_DEFAULT GEDIT_DOCUMENT_NEWLINE_TYPE_CR_LF
+#else
+#define GEDIT_DOCUMENT_NEWLINE_TYPE_DEFAULT GEDIT_DOCUMENT_NEWLINE_TYPE_LF
+#endif
+
 typedef enum
 {
 	GEDIT_SEARCH_DONT_SET_FLAGS	= 1 << 0, 
@@ -139,10 +162,10 @@ struct _GeditDocumentClass
 
 enum
 {
-	/* we use GIO error codes, so start at some high number? */
-	GEDIT_DOCUMENT_ERROR_EXTERNALLY_MODIFIED = 1 << 10,
+	GEDIT_DOCUMENT_ERROR_EXTERNALLY_MODIFIED,
 	GEDIT_DOCUMENT_ERROR_CANT_CREATE_BACKUP,
 	GEDIT_DOCUMENT_ERROR_TOO_BIG,
+	GEDIT_DOCUMENT_ERROR_CONVERSION_FALLBACK,
 	GEDIT_DOCUMENT_NUM_ERRORS 
 };
 
@@ -163,6 +186,9 @@ gchar		*gedit_document_get_uri_for_display
 gchar		*gedit_document_get_short_name_for_display
 					 	(GeditDocument       *doc);
 
+gchar		*gedit_document_get_content_type
+					 	(GeditDocument       *doc);
+
 gchar		*gedit_document_get_mime_type 	(GeditDocument       *doc);
 
 gboolean	 gedit_document_get_readonly 	(GeditDocument       *doc);
@@ -198,6 +224,10 @@ gboolean	 gedit_document_get_deleted	(GeditDocument       *doc);
 gboolean	 gedit_document_goto_line 	(GeditDocument       *doc, 
 						 gint                 line);
 
+gboolean	 gedit_document_goto_line_offset(GeditDocument *doc,
+						 gint           line,
+						 gint           line_offset);
+
 void		 gedit_document_set_search_text	(GeditDocument       *doc,
 						 const gchar         *text,
 						 guint                flags);
@@ -240,10 +270,25 @@ void		 gedit_document_set_enable_search_highlighting
 gboolean	 gedit_document_get_enable_search_highlighting
 						(GeditDocument       *doc);
 
-		
+void		 gedit_document_set_newline_type (GeditDocument           *doc,
+						  GeditDocumentNewlineType newline_type);
+
+GeditDocumentNewlineType
+		 gedit_document_get_newline_type (GeditDocument *doc);
+
+gchar		*gedit_document_get_metadata	(GeditDocument *doc,
+						 const gchar   *key);
+
+void		 gedit_document_set_metadata	(GeditDocument *doc,
+						 const gchar   *first_key,
+						 ...);
+
 /* 
  * Non exported functions
  */
+void		 _gedit_document_set_readonly 	(GeditDocument       *doc,
+						 gboolean             readonly);
+
 glong		 _gedit_document_get_seconds_since_last_save_or_load 
 						(GeditDocument       *doc);
 
