@@ -468,76 +468,57 @@ static void
 io_loading_error_message_area_response (GtkWidget        *message_area,
 					gint              response_id,
 					GeditTab         *tab)
-{
-	if (response_id == GTK_RESPONSE_OK)
-	{
-		GeditDocument *doc;
-		gchar *uri;
-
-		doc = gedit_tab_get_document (tab);
-		g_return_if_fail (GEDIT_IS_DOCUMENT (doc));
-
-		uri = gedit_document_get_uri (doc);
-		g_return_if_fail (uri != NULL);
-
-		set_message_area (tab, NULL);
-		gedit_tab_set_state (tab, GEDIT_TAB_STATE_LOADING);
-
-		g_return_if_fail (tab->priv->auto_save_timeout <= 0);
-
-		gedit_document_load (doc,
-				     uri,
-				     tab->priv->tmp_encoding,
-				     tab->priv->tmp_line_pos,
-				     FALSE);
-	}
-	else
-	{
-		remove_tab (tab);
-	}
-}
-
-static void 
-conversion_loading_error_message_area_response (GtkWidget        *message_area,
-						gint              response_id,
-						GeditTab         *tab)
 {
 	GeditDocument *doc;
+	GeditView *view;
 	gchar *uri;
+	const GeditEncoding *encoding;
 
 	doc = gedit_tab_get_document (tab);
 	g_return_if_fail (GEDIT_IS_DOCUMENT (doc));
 
+	view = gedit_tab_get_view (tab);
+	g_return_if_fail (GEDIT_IS_VIEW (view));
+
 	uri = gedit_document_get_uri (doc);
 	g_return_if_fail (uri != NULL);
 
-	if (response_id == GTK_RESPONSE_OK)
+	switch (response_id)
 	{
-		const GeditEncoding *encoding;
+		case GTK_RESPONSE_OK:
+			encoding = gedit_conversion_error_message_area_get_encoding (
+					GTK_WIDGET (message_area));
 
-		encoding = gedit_conversion_error_message_area_get_encoding (
-				GTK_WIDGET (message_area));
+			if (encoding != NULL)
+			{
+				tab->priv->tmp_encoding = encoding;
+			}
 
-		g_return_if_fail (encoding != NULL);
+			set_message_area (tab, NULL);
+			gedit_tab_set_state (tab, GEDIT_TAB_STATE_LOADING);
 
-		set_message_area (tab, NULL);
-		gedit_tab_set_state (tab, GEDIT_TAB_STATE_LOADING);
+			g_return_if_fail (tab->priv->auto_save_timeout <= 0);
 
-		tab->priv->tmp_encoding = encoding;
+			gedit_document_load (doc,
+					     uri,
+					     tab->priv->tmp_encoding,
+					     tab->priv->tmp_line_pos,
+					     FALSE);
+			break;
+		case GTK_RESPONSE_YES:
+			/* This means that we want to edit the document anyway */
+			set_message_area (tab, NULL);
+			_gedit_document_set_readonly (doc, FALSE);
+			break;
+		case GTK_RESPONSE_NO:
+			/* We don't want to edit the document just show it */
+			set_message_area (tab, NULL);
+			break;
+		default:
+			_gedit_recent_remove (GEDIT_WINDOW (gtk_widget_get_toplevel (GTK_WIDGET (tab))), uri);
 
-		g_return_if_fail (tab->priv->auto_save_timeout <= 0);
-
-		gedit_document_load (doc,
-				     uri,
-				     encoding,
-				     tab->priv->tmp_line_pos,
-				     FALSE);
-	}
-	else
-	{
-		_gedit_recent_remove (GEDIT_WINDOW (gtk_widget_get_toplevel (GTK_WIDGET (tab))), uri);
-
-		remove_tab (tab);
+			remove_tab (tab);
+			break;
 	}
 
 	g_free (uri);
@@ -927,7 +908,8 @@ document_loaded (GeditDocument *document,
 	location = gedit_document_get_location (document);
 	uri = gedit_document_get_uri (document);
 
-	if (error != NULL)
+	/* if the error is CONVERSION FALLBACK don't treat it as a normal error */
+	if (error != NULL && error->code != GEDIT_DOCUMENT_ERROR_CONVERSION_FALLBACK)
 	{
 		if (tab->priv->state == GEDIT_TAB_STATE_LOADING)
 			gedit_tab_set_state (tab, GEDIT_TAB_STATE_LOADING_ERROR);
@@ -947,14 +929,14 @@ document_loaded (GeditDocument *document,
 
 			goto end;
 		}
-		else if (error->domain == G_IO_ERROR || 
-			 error->domain == GEDIT_DOCUMENT_ERROR)
+		else
 		{
 			_gedit_recent_remove (GEDIT_WINDOW (gtk_widget_get_toplevel (GTK_WIDGET (tab))), uri);
 
 			if (tab->priv->state == GEDIT_TAB_STATE_LOADING_ERROR)
 			{
-				emsg = gedit_io_loading_error_message_area_new (uri, 
+				emsg = gedit_io_loading_error_message_area_new (uri,
+										tab->priv->tmp_encoding,
 										error);
 				g_signal_connect (emsg,
 						  "response",
@@ -965,7 +947,7 @@ document_loaded (GeditDocument *document,
 			{
 				g_return_if_fail (tab->priv->state == GEDIT_TAB_STATE_REVERTING_ERROR);
 				
-				emsg = gedit_unrecoverable_reverting_error_message_area_new (uri, 
+				emsg = gedit_unrecoverable_reverting_error_message_area_new (uri,
 											     error);
 
 				g_signal_connect (emsg,
@@ -976,25 +958,6 @@ document_loaded (GeditDocument *document,
 
 			set_message_area (tab, emsg);
 		}
-		else
-		{
-			g_return_if_fail ((error->domain == G_CONVERT_ERROR) ||
-			      		  (error->domain == GEDIT_CONVERT_ERROR));
-
-			// TODO: different error messages if tab->priv->state == GEDIT_TAB_STATE_REVERTING?
-			// note that while reverting encoding should be ok, so this is unlikely to happen
-			emsg = gedit_conversion_error_while_loading_message_area_new (
-									uri,
-									tab->priv->tmp_encoding,
-									error);
-
-			set_message_area (tab, emsg);
-
-			g_signal_connect (emsg,
-					  "response",
-					  G_CALLBACK (conversion_loading_error_message_area_response),
-					  tab);
-		}
 
 #if !GTK_CHECK_VERSION (2, 17, 1)
 		gedit_message_area_set_default_response (GEDIT_MESSAGE_AREA (emsg),
@@ -1025,6 +988,37 @@ document_loaded (GeditDocument *document,
 				   mime);
 		g_free (mime);
 
+		if (error && error->code == GEDIT_DOCUMENT_ERROR_CONVERSION_FALLBACK)
+		{
+			GtkWidget *emsg;
+
+			_gedit_document_set_readonly (document, TRUE);
+
+			emsg = gedit_io_loading_error_message_area_new (uri,
+									tab->priv->tmp_encoding,
+									error);
+
+			set_message_area (tab, emsg);
+
+			g_signal_connect (emsg,
+					  "response",
+					  G_CALLBACK (io_loading_error_message_area_response),
+					  tab);
+
+#if !GTK_CHECK_VERSION (2, 17, 1)
+			gedit_message_area_set_default_response (GEDIT_MESSAGE_AREA (emsg),
+								 GTK_RESPONSE_CANCEL);
+#else
+			gtk_info_bar_set_default_response (GTK_INFO_BAR (emsg),
+							   GTK_RESPONSE_CANCEL);
+#endif
+
+			gtk_widget_show (emsg);
+		}
+
+		/* Scroll to the cursor when the document is loaded */
+		gedit_view_scroll_to_cursor (GEDIT_VIEW (tab->priv->view));
+
 		all_documents = gedit_app_get_documents (gedit_app_get_default ());
 
 		for (l = all_documents; l != NULL; l = g_list_next (l))
@@ -2183,9 +2177,10 @@ gedit_tab_auto_save (GeditTab *tab)
 }
 
 void
-_gedit_tab_save_as (GeditTab            *tab,
-		    const gchar         *uri,
-		    const GeditEncoding *encoding)
+_gedit_tab_save_as (GeditTab                 *tab,
+                    const gchar              *uri,
+                    const GeditEncoding      *encoding,
+                    GeditDocumentNewlineType  newline_type)
 {
 	GeditDocument *doc;
 	GeditDocumentSaveFlags save_flags;
@@ -2230,6 +2225,11 @@ _gedit_tab_save_as (GeditTab            *tab,
 	if (tab->priv->auto_save_timeout > 0)
 		remove_auto_save_timeout (tab);
 
+	/* FIXME: this should behave the same as encoding, setting it here
+	   makes it persistent (if save fails, it's remembered). It's not
+	   a very big deal, but would be nice to have them follow the
+	   same pattern. This can be changed once we break API for 3.0 */
+	gedit_document_set_newline_type (doc, newline_type);
 	gedit_document_save_as (doc, uri, encoding, tab->priv->save_flags);
 }
 
@@ -2295,6 +2295,46 @@ printing_cb (GeditPrintJob       *job,
 						  gedit_print_job_get_progress (job));
 }
 
+static void
+store_print_settings (GeditTab      *tab,
+		      GeditPrintJob *job)
+{
+	GeditDocument *doc;
+	GtkPrintSettings *settings;
+	GtkPageSetup *page_setup;
+
+	doc = gedit_tab_get_document (tab);
+
+	settings = gedit_print_job_get_print_settings (job);
+
+	/* clear n-copies settings since we do not want to
+	 * persist that one */
+	gtk_print_settings_unset (settings,
+				  GTK_PRINT_SETTINGS_N_COPIES);
+
+	/* remember settings for this document */
+	g_object_set_data_full (G_OBJECT (doc),
+				GEDIT_PRINT_SETTINGS_KEY,
+				g_object_ref (settings),
+				(GDestroyNotify)g_object_unref);
+
+	/* make them the default */
+	_gedit_app_set_default_print_settings (gedit_app_get_default (),
+					       settings);
+
+	page_setup = gedit_print_job_get_page_setup (job);
+
+	/* remember page setup for this document */
+	g_object_set_data_full (G_OBJECT (doc),
+				GEDIT_PAGE_SETUP_KEY,
+				g_object_ref (page_setup),
+				(GDestroyNotify)g_object_unref);
+
+	/* make it the default */
+	_gedit_app_set_default_page_setup (gedit_app_get_default (),
+					   page_setup);
+}
+
 static void
 done_printing_cb (GeditPrintJob       *job,
 		  GeditPrintJobResult  result,
@@ -2321,38 +2361,9 @@ done_printing_cb (GeditPrintJob       *job,
 
 	// TODO: check status and error
 
-	/* Save the print settings and the page setup */ 
 	if (result ==  GEDIT_PRINT_JOB_RESULT_OK)
 	{
-		GeditDocument *doc;
-		GtkPrintSettings *settings;
-		GtkPageSetup *page_setup;
-
-		doc = gedit_tab_get_document (tab);
-
-		settings = gedit_print_job_get_print_settings (job);
-
-		/* remember settings for this document */
-		g_object_set_data_full (G_OBJECT (doc),
-					GEDIT_PRINT_SETTINGS_KEY,
-					g_object_ref (settings),
-					(GDestroyNotify)g_object_unref);
-
-		/* make them the default */
-		_gedit_app_set_default_print_settings (gedit_app_get_default (),
-						       settings);
-
-		page_setup = gedit_print_job_get_page_setup (job);
-
-		/* remember page setup for this document */
-		g_object_set_data_full (G_OBJECT (doc),
-					GEDIT_PAGE_SETUP_KEY,
-					g_object_ref (page_setup),
-					(GDestroyNotify)g_object_unref);
-
-		/* make it the default */
-		_gedit_app_set_default_page_setup (gedit_app_get_default (),
-						   page_setup);
+		store_print_settings (tab, job);
 	}
 
 #if 0
